@@ -5,4 +5,4 @@ License for this software, part of the pyDE1 package, is granted under
 GNU General Public License v3.0 only
 SPDX-License-Identifier: GPL-3.0-only
 """
-from .run import run_api_outbound
+from .run import run_mqtt_outbound, OutboundMode
