@@ -9,68 +9,29 @@ Provide a mapping from internal objects and properties, sufficient for both
 creation of outgoing dict, as well as validation of incoming requests
 (Resource, key, target object, property name, value type, min, max, post_execute)
 
-Provide "mock" classes for use in inbound API process
-
-NB: Must be imported AFTER any imports of DE1, Scale, FlowSequencer, ...
+As of MAPPING_VERSION 4.0.0 direct use of the class or a stub has been replaed
+by use of an enum. Consumers will need to map the enum to the class.
+This is due to challenges in determining if the class or the stub is needed.
 """
+
 import importlib.util
 import inspect
-import logging
-import multiprocessing
 import sys
+from enum import Enum, auto
 from typing import Optional, Union, NamedTuple
 
-from pyDE1.dispatcher.resource import Resource, RESOURCE_VERSION, \
-    DE1ModeEnum, ConnectivityEnum
+import pyDE1
+from pyDE1.de1.c_api import (
+    PackedAttr, MMR0x80LowAddr,
+    SetTime, ShotSettings, Versions, WaterLevels,
+)
+from pyDE1.dispatcher.resource import (
+    Resource, RESOURCE_VERSION, DE1ModeEnum, ConnectivityEnum
+)
 
-# TODO: Work through main and remote thread imports
+MAPPING_VERSION = "4.0.1"
 
-logger = logging.getLogger('Mapping')
-
-pname = f"into {multiprocessing.current_process().name} process"
-
-if 'pyDE1.de1' in sys.modules:
-    logger.info("Importing DE1")
-    from pyDE1.de1 import DE1
-else:
-    logger.info("Using stub for DE1")
-    from pyDE1.dispatcher.stubs import DE1
-
-if 'pyDE1.scale' in sys.modules:
-    logger.info("Importing Scale")
-    from pyDE1.scale import Scale
-else:
-    logger.info("Using stub for Scale")
-    from pyDE1.dispatcher.stubs import Scale
-
-if 'pyDE1.scale.processor' in sys.modules:
-    logger.info("Importing ScaleProcessor")
-    from pyDE1.scale.processor import ScaleProcessor
-else:
-    logger.info("Using stub for ScaleProcessor")
-    from pyDE1.dispatcher.stubs import ScaleProcessor
-
-if 'pyDE1.flow_sequencer' in sys.modules:
-    logger.info("Importing FlowSequencer")
-    from pyDE1.flow_sequencer import FlowSequencer
-else:
-    logger.info("Using stub for FlowSequencer")
-    from pyDE1.dispatcher.stubs import FlowSequencer
-
-if 'pyDE1.scanner' in sys.modules:
-    logger.info("Importing DiscoveredDevices")
-    from pyDE1.scanner import DiscoveredDevices
-else:
-    logger.info("Using stub for DiscoveredDevices")
-    from pyDE1.dispatcher.stubs import DiscoveredDevices
-
-
-from pyDE1.de1.c_api import PackedAttr, MMR0x80LowAddr, get_cuuid, \
-    ShotSettings, SetTime, Versions, WaterLevels
-
-from pyDE1.exceptions import DE1APIValueError
-
-MAPPING_VERSION = "3.1.0"
+logger = pyDE1.getLogger('Inbound.Mapping')
 
 # There are a handful of requests related to the DE1 and Scale
 # that can be or must be handled even if the device is not ready.
@@ -82,13 +43,20 @@ MAPPING_VERSION = "3.1.0"
 #   * Add the "if_not_ready" element to IsAt and check in requires_*
 
 
+class TO (Enum):
+    DE1 = auto()
+    DiscoveredDevices = auto()
+    FlowSequencer = auto()
+    Scale = auto()
+    ScaleProcessor = auto()
+    
 
-class IsAt (NamedTuple):
-    target: Union[type(DE1),
-                  type(Scale),
-                  type(FlowSequencer),
+class IsAt(NamedTuple):
+    # TODO: This list appears to miss None for Resource.SCAN (scanner.py)
+    #       Maybe reference by the module?
+    target: Union[type(TO),
                   type(PackedAttr),
-                  type(MMR0x80LowAddr)]
+                  type(MMR0x80LowAddr),]
     attr_path: Optional[str]  # '' for MMR0x80LowAddr, None for write_only,
     v_type: type  # expected value type for type-checking
     setter_path: Optional[str] = None   # If not a property and a different path
@@ -104,7 +72,7 @@ class IsAt (NamedTuple):
     @property
     def requires_connected_de1(self) -> bool:
         retval = (
-            (self.target is DE1 and not self.if_not_ready)
+            (self.target == TO.DE1 and not self.if_not_ready)
             or isinstance(self.target, MMR0x80LowAddr)
             or (inspect.isclass(self.target)
                 and issubclass(self.target, PackedAttr))
@@ -114,7 +82,7 @@ class IsAt (NamedTuple):
     @property
     def requires_connected_scale(self) -> bool:
         retval = (
-            (self.target is Scale and not self.if_not_ready)
+            (self.target == TO.Scale and not self.if_not_ready)
         )
         return retval
 
@@ -128,7 +96,6 @@ def mapping_requires(mapping: dict) -> dict:
 
 
 def _mapping_requires_inner(mapping: dict, results: dict) -> dict:
-
     for val in mapping.values():
         if isinstance(val, IsAt):
             if val.requires_connected_de1:
@@ -160,11 +127,11 @@ def from_packed_attr(packed_attr: PackedAttr, attr_path: str, v_type: type,
         )
 
     return IsAt(
-        target= packed_attr,
-        attr_path= attr_path,
-        v_type= v_type,
-        setter_path= setter_path,
-        read_only= not can_read,
+        target=packed_attr,
+        attr_path=attr_path,
+        v_type=v_type,
+        setter_path=setter_path,
+        read_only=not can_read,
     )
 
 
@@ -172,24 +139,27 @@ def from_packed_attr(packed_attr: PackedAttr, attr_path: str, v_type: type,
 def from_mmr(mmr: MMR0x80LowAddr, v_type: type):
     # This doesn't handle can't read and can't write
     return IsAt(
-        target= mmr,
-        attr_path= '',
-        v_type= v_type,
-        read_only= not mmr.can_read
+        target=mmr,
+        attr_path='',
+        v_type=v_type,
+        read_only=not mmr.can_read
     )
 
 
 MODULES_FOR_VERSIONS = (
     'pyDE1',
-    'bleak',
     'aiosqlite',
+    'bleak',
+    'paho-mqtt',
+    'PyYAML',
+    'requests',
+    # Retained for backward compatibility - consider as deprecated in this list
     'asyncio-mqtt',
-    'paho-mqtt'
 )
 
-
 import importlib.metadata  # Used for module-version lookup only
 
+
 def module_versions():
     retval = {}
     for module in MODULES_FOR_VERSIONS:
@@ -219,10 +189,18 @@ MAPPING[Resource.VERSION] = {
 
 # "Specials" -- content-only
 
-MAPPING[Resource.DE1_PROFILE] = IsAt(target=DE1, attr_path=None,
+MAPPING[Resource.DE1_PROFILE] = IsAt(target=TO.DE1, attr_path=None,
                                      setter_path='upload_json_v2_profile',
                                      v_type=Union[bytes, bytearray])
 
+MAPPING[Resource.DE1_FIRMWARE] = IsAt(target=TO.DE1, attr_path=None,
+                                      setter_path='upload_firmware_from_content',
+                                      v_type=Union[bytes, bytearray])
+
+MAPPING[Resource.DE1_FIRMWARE_CANCEL] = IsAt(target=TO.DE1, attr_path=None,
+                                             setter_path='cancel_firmware_api',
+                                             v_type=Union[bytes, bytearray])
+
 # TODO: How to reference a module-level subroutine? Use module as target?
 # TODO: How to get the scan ID back to the caller
 
@@ -231,9 +209,9 @@ MAPPING[Resource.SCAN] = {
                   setter_path='scan_from_api', v_type=bool)
 }
 
-# TODO: This is an async getter because of the lock
+# Note: This is an async getter because of the lock
 MAPPING[Resource.SCAN_DEVICES] = {
-    'devices': IsAt(target=DiscoveredDevices, attr_path='devices_for_json',
+    'devices': IsAt(target=TO.DiscoveredDevices, attr_path='devices_for_json',
                     read_only=True,
                     v_type=dict)
 }
@@ -242,100 +220,105 @@ MAPPING[Resource.SCAN_DEVICES] = {
 
 # For now, name is not writable
 MAPPING[Resource.DE1_ID] = {
-    'name': IsAt(target=DE1, attr_path='name', v_type=Optional[str],
+    'name': IsAt(target=TO.DE1, attr_path='name', v_type=Optional[str],
                  read_only=True,
                  if_not_ready=True),
-    'id': IsAt(target=DE1, attr_path='address', v_type=Optional[str],
+    'id': IsAt(target=TO.DE1, attr_path='address', v_type=Optional[str],
                setter_path='change_de1_to_id',
                if_not_ready=True),
     # first_if_found, if true, will replace only if one is found
     # It is an error to be true if 'id' is present at this time
-    'first_if_found': IsAt(target=DE1, attr_path=None,
-                        setter_path='first_if_found', v_type=bool,
+    'first_if_found': IsAt(target=TO.DE1, attr_path=None,
+                           setter_path='first_if_found', v_type=bool,
                            if_not_ready=True),
 }
 
 # NB: A single-entry tuple needs to end with a comma
 
 MAPPING[Resource.DE1_MODE] = {
-    'mode': IsAt(target=DE1, attr_path=None, setter_path='mode_setter',
+    'mode': IsAt(target=TO.DE1, attr_path=None, setter_path='mode_setter',
                  v_type=str, internal_type=DE1ModeEnum),
 }
-# TODO: de1.mode()
+
+MAPPING[Resource.DE1_STATE] = {
+    'state': IsAt(target=TO.DE1, attr_path='state_getter',
+                  read_only=True,
+                  v_type=str, internal_type=str),
+}
+
+MAPPING[Resource.DE1_FEATURE_FLAGS] = {
+    'feature_flags': IsAt(target=TO.DE1, attr_path='feature_flags',
+                          v_type=str, read_only=True),
+}
 
 # DE1_PROFILE = 'de1/profile'
 # DE1_PROFILES = 'de1/profiles'
 
 MAPPING[Resource.DE1_PROFILE_ID] = {
-    'id': IsAt(target=DE1, attr_path='profile_id', v_type=str,
+    'id': IsAt(target=TO.DE1, attr_path='profile_id', v_type=str,
                setter_path='set_profile_by_id')
 }
 
-
 # DE1_FIRMWARE = 'de1/firmware'
 # DE1_FIRMWARES = 'de1/firmwares'
 
 MAPPING[Resource.DE1_FIRMWARE_ID] = {
-    'id': IsAt(target=MMR0x80LowAddr.FIRMWARE_BUILD_NUMBER,
+    'id': IsAt(target=MMR0x80LowAddr.CPU_FIRMWARE_BUILD,
                attr_path='', v_type=int),
 }
 
-
-
 MAPPING[Resource.DE1_CONNECTIVITY] = {
-    'mode': IsAt(target=DE1, attr_path='connectivity',
+    'mode': IsAt(target=TO.DE1, attr_path='connectivity',
                  setter_path='connectivity_setter', v_type=str,
                  internal_type=ConnectivityEnum,
                  if_not_ready=True),
 }
 
 # DE1_CONTROL = 'de1/control' -- aggregate
+# TODO: Work through how to get the aggregate to operate
 
-# TODO: Work through how to get this to work
-
-# TODO: Allow stop_at_xxxx for everything, None means don't apply
 
 MAPPING[Resource.DE1_CONTROL_ESPRESSO] = {
-    'stop_at_time': IsAt(target=FlowSequencer,
+    'stop_at_time': IsAt(target=TO.FlowSequencer,
                          attr_path='espresso_control.stop_at_time',
                          v_type=Optional[float]),
-    'stop_at_volume': IsAt(target=FlowSequencer,
+    'stop_at_volume': IsAt(target=TO.FlowSequencer,
                            attr_path='espresso_control.stop_at_volume',
                            v_type=Optional[float]),
-    'stop_at_weight': IsAt(target=FlowSequencer,
+    'stop_at_weight': IsAt(target=TO.FlowSequencer,
                            attr_path='espresso_control.stop_at_weight',
                            v_type=Optional[float]),
-    'disable_auto_tare': IsAt(target=FlowSequencer,
+    'disable_auto_tare': IsAt(target=TO.FlowSequencer,
                               attr_path='espresso_control.disable_auto_tare',
                               v_type=bool),
 
     'profile_can_override_stop_limits':
-        IsAt(target=FlowSequencer,
+        IsAt(target=TO.FlowSequencer,
              attr_path='espresso_control.profile_can_override_stop_limits',
              v_type=bool),
     'profile_can_override_tank_temperature':
-        IsAt(target=FlowSequencer,
+        IsAt(target=TO.FlowSequencer,
              attr_path='espresso_control.profile_can_override_tank_temperature',
              v_type=bool),
     'first_drops_threshold':
-        IsAt(target=FlowSequencer,
+        IsAt(target=TO.FlowSequencer,
              attr_path='espresso_control.first_drops_threshold',
              v_type=Optional[float]),
     'last_drops_minimum_time':
-        IsAt(target=FlowSequencer,
+        IsAt(target=TO.FlowSequencer,
              attr_path='espresso_control.last_drops_minimum_time',
              v_type=float),
 }
 
 MAPPING[Resource.DE1_CONTROL_STEAM] = {
     'stop_at_time': IsAt(target=ShotSettings, attr_path='TargetSteamLength', v_type=int),
-    'stop_at_volume': IsAt(target=FlowSequencer,
+    'stop_at_volume': IsAt(target=TO.FlowSequencer,
                            attr_path='steam_control.stop_at_volume',
                            v_type=Optional[float]),
-    'stop_at_weight': IsAt(target=FlowSequencer,
+    'stop_at_weight': IsAt(target=TO.FlowSequencer,
                            attr_path='steam_control.stop_at_weight',
                            v_type=Optional[float]),
-    'disable_auto_tare': IsAt(target=FlowSequencer,
+    'disable_auto_tare': IsAt(target=TO.FlowSequencer,
                               attr_path='steam_control.disable_auto_tare',
                               v_type=bool),
 }
@@ -347,10 +330,10 @@ MAPPING[Resource.DE1_CONTROL_HOT_WATER] = {
     'stop_at_volume': IsAt(target=ShotSettings,
                            attr_path='TargetHotWaterVol',
                            v_type=int),
-    'stop_at_weight': IsAt(target=FlowSequencer,
+    'stop_at_weight': IsAt(target=TO.FlowSequencer,
                            attr_path='hot_water_control.stop_at_weight',
                            v_type=Optional[float]),
-    'disable_auto_tare': IsAt(target=FlowSequencer,
+    'disable_auto_tare': IsAt(target=TO.FlowSequencer,
                               attr_path='hot_water_control.disable_auto_tare',
                               v_type=bool),
     'temperature': IsAt(target=ShotSettings,
@@ -359,30 +342,35 @@ MAPPING[Resource.DE1_CONTROL_HOT_WATER] = {
 
 MAPPING[Resource.DE1_CONTROL_HOT_WATER_RINSE] = {
     'stop_at_time':
-        IsAt(target=FlowSequencer,
+        IsAt(target=TO.FlowSequencer,
              attr_path='hot_water_rinse_control.stop_at_time',
+             setter_path='hot_water_rinse_control.stop_at_time_set_async',
              v_type=Optional[float]),
     'stop_at_volume':
-        IsAt(target=FlowSequencer,
+        IsAt(target=TO.FlowSequencer,
              attr_path='hot_water_rinse_control.stop_at_volume',
              v_type=Optional[float]),
     'stop_at_weight':
-        IsAt(target=FlowSequencer,
+        IsAt(target=TO.FlowSequencer,
              attr_path='hot_water_rinse_control.stop_at_weight',
              v_type=Optional[float]),
     'disable_auto_tare':
-        IsAt(target=FlowSequencer,
+        IsAt(target=TO.FlowSequencer,
              attr_path='hot_water_rinse_control.disable_auto_tare', v_type=bool),
+    'temperature':
+        IsAt(target=MMR0x80LowAddr.FLUSH_TEMP, attr_path='', v_type=float),
+    'flow':
+        IsAt(target=MMR0x80LowAddr.FLUSH_FLOW_RATE, attr_path='', v_type=float),
 }
 
 MAPPING[Resource.DE1_CONTROL_TANK_WATER_THRESHOLD] = {
-    'temperature': IsAt(target=MMR0x80LowAddr.TANK_WATER_THRESHOLD, attr_path='', v_type=int),
+    'temperature': IsAt(target=MMR0x80LowAddr.TANK_TEMP, attr_path='', v_type=int),
 }
 
 # DE1_SETTING = 'de1/setting' -- aggregate
 
 MAPPING[Resource.DE1_SETTING_AUTO_OFF_TIME] = {
-    'time': IsAt(target=DE1, attr_path='auto_off_time',
+    'time': IsAt(target=TO.DE1, attr_path='auto_off_time',
                  v_type=Optional[float]),
 }
 
@@ -395,19 +383,20 @@ MAPPING[Resource.DE1_SETTING_START_FILL_LEVEL] = {
 }
 
 MAPPING[Resource.DE1_SETTING_BEFORE_FLOW] = {
-    'heater_phase1_flow': IsAt(target=MMR0x80LowAddr.HEATER_PHASE1_FLOW, attr_path='', v_type=float),
-    'heater_phase2_flow': IsAt(target=MMR0x80LowAddr.HEATER_PHASE2_FLOW, attr_path='', v_type=float),
-    'heater_phase2_timeout': IsAt(target=MMR0x80LowAddr.HEATER_PHASE2_TIMEOUT, attr_path='', v_type=float),
-    'heater_idle_temperature': IsAt(target=MMR0x80LowAddr.HEATER_IDLE_TEMPERATURE, attr_path='', v_type=float),
+    'heater_phase1_flow': IsAt(target=MMR0x80LowAddr.HEATER_UP1_FLOW, attr_path='', v_type=float),
+    'heater_phase2_flow': IsAt(target=MMR0x80LowAddr.HEATER_UP2_FLOW, attr_path='', v_type=float),
+    'heater_phase2_timeout': IsAt(target=MMR0x80LowAddr.HEATER_UP2_TIMEOUT, attr_path='', v_type=float),
+    'heater_idle_temperature': IsAt(target=MMR0x80LowAddr.WATER_HEATER_IDLE_TEMP, attr_path='', v_type=float),
 }
 
 MAPPING[Resource.DE1_SETTING_STEAM] = {
     'temperature': IsAt(target=ShotSettings, attr_path='TargetSteamTemp', v_type=int),
-    'flow': IsAt(target=MMR0x80LowAddr.STEAM_FLOW_RATE, attr_path='', v_type=float),
-    'high_flow_time': IsAt(target=MMR0x80LowAddr.HIGH_STEAM_FLOW_TIME, attr_path='', v_type=float),
+    'flow': IsAt(target=MMR0x80LowAddr.TARGET_STEAM_FLOW, attr_path='', v_type=float),
+    'high_flow_time': IsAt(target=MMR0x80LowAddr.STEAM_START_SECS, attr_path='', v_type=float),
 }
 
-# TODO: What is ShotSettings.TargetGroupTemp and where does it really belong?
+# TODO: What is ShotSettings.TargetGroupTemp in the current DE1 firmware
+#       and where does it really belong?
 MAPPING[Resource.DE1_SETTING_TARGET_GROUP_TEMP] = {
     'temperature': IsAt(target=ShotSettings, attr_path='TargetGroupTemp', v_type=int),
 }
@@ -434,8 +423,8 @@ MAPPING[Resource.DE1_READ_ONCE] = {
     'hw_config_hexstr': IsAt(target=MMR0x80LowAddr.HW_CONFIG, attr_path='', v_type=str),
     'model_hexstr': IsAt(target=MMR0x80LowAddr.MODEL, attr_path='', v_type=str),
     'cpu_board_model': IsAt(target=MMR0x80LowAddr.CPU_BOARD_MODEL, attr_path='', v_type=float),
-    'firmware_model': IsAt(target=MMR0x80LowAddr.FIRMWARE_MODEL, attr_path='', v_type=str),
-    'firmware_build_number': IsAt(target=MMR0x80LowAddr.FIRMWARE_BUILD_NUMBER, attr_path='', v_type=int),
+    'firmware_model': IsAt(target=MMR0x80LowAddr.V13_MODEL, attr_path='', v_type=str),
+    'firmware_build_number': IsAt(target=MMR0x80LowAddr.CPU_FIRMWARE_BUILD, attr_path='', v_type=int),
     'ghc_info': IsAt(target=MMR0x80LowAddr.GHC_INFO, attr_path='', v_type=str),  # See MMRGHCInfoBitMask
     'serial_number_hexstr': IsAt(target=MMR0x80LowAddr.SERIAL_NUMBER, attr_path='', v_type=str),
     'heater_voltage': IsAt(target=MMR0x80LowAddr.HEATER_VOLTAGE, attr_path='', v_type=int),
@@ -458,48 +447,48 @@ MAPPING[Resource.DE1_READ_ONCE] = {
 }
 
 MAPPING[Resource.DE1_CALIBRATION_FLOW_MULTIPLIER] = {
-    'multiplier': IsAt(target=MMR0x80LowAddr.FLOW_CALIBRATION,
+    'multiplier': IsAt(target=MMR0x80LowAddr.CAL_FLOW_EST,
                        attr_path='', v_type=float),
 }
 
 MAPPING[Resource.DE1_CALIBRATION_LINE_FREQUENCY] = {
-    'hz': IsAt(target=DE1, attr_path='line_frequency', v_type=int)
+    'hz': IsAt(target=TO.DE1, attr_path='line_frequency', v_type=int)
 }
 
 MAPPING[Resource.SCALE_ID] = {
-    'name': IsAt(target=ScaleProcessor, attr_path='scale_name',
+    'name': IsAt(target=TO.ScaleProcessor, attr_path='scale_name',
                  v_type=Optional[str],
                  read_only=True,
                  if_not_ready=True),
-    'id': IsAt(target=ScaleProcessor, attr_path='scale_address',
+    'id': IsAt(target=TO.ScaleProcessor, attr_path='scale_address',
                v_type=Optional[str],
                setter_path='change_scale_to_id',
                if_not_ready=True),
-    'type': IsAt(target=ScaleProcessor, attr_path='scale_type',
+    'type': IsAt(target=TO.ScaleProcessor, attr_path='scale_type',
                  v_type=Optional[str],
                  read_only=True,
                  if_not_ready=True),
     # first_if_found, if true, will replace only if one is found
     # It is an error to be true if 'id' is present at this time
-    'first_if_found': IsAt(target=ScaleProcessor, attr_path=None,
+    'first_if_found': IsAt(target=TO.ScaleProcessor, attr_path=None,
                            setter_path='first_if_found', v_type=bool,
                            if_not_ready=True),
 }
 
 MAPPING[Resource.SCALE_CONNECTIVITY] = {
-    'mode': IsAt(target=ScaleProcessor, attr_path='scale_connectivity',
+    'mode': IsAt(target=TO.ScaleProcessor, attr_path='scale_connectivity',
                  setter_path="connectivity_setter", v_type=str,
                  internal_type=ConnectivityEnum,
                  if_not_ready=True),
 }
 
 MAPPING[Resource.SCALE_TARE] = {
-    'tare': IsAt(target=Scale, attr_path='', setter_path='tare_with_bool',
-                 v_type=Optional[bool])     # Accommodate None as False
+    'tare': IsAt(target=TO.Scale, attr_path='', setter_path='tare_with_bool',
+                 v_type=Optional[bool])  # Accommodate None as False
 }
 
 MAPPING[Resource.SCALE_DISPLAY] = {
-    'display_on': IsAt(target=Scale, attr_path='', setter_path='display_bool',
+    'display_on': IsAt(target=TO.Scale, attr_path='', setter_path='display_bool',
                        v_type=Optional[bool])  # Accommodate None as False
 }
 
@@ -527,10 +516,10 @@ MAPPING[Resource.DE1_CALIBRATION] = {
     # 'internal': Mapping[Resource.DE1_CALIBRATION_INTERNAL],
 }
 
-# TODO: How to handle GET if non-GET items? NaN? null?
 MAPPING[Resource.DE1] = {
     'id': MAPPING[Resource.DE1_ID],
     'mode': MAPPING[Resource.DE1_MODE],
+    'state': MAPPING[Resource.DE1_STATE],
     # profile
     # profiles
     # firmware
@@ -543,7 +532,6 @@ MAPPING[Resource.DE1] = {
     'read_once': MAPPING[Resource.DE1_READ_ONCE],
 }
 
-
 MAPPING[Resource.SCALE] = {
     'id': MAPPING[Resource.SCALE_ID],
     'connectivity': MAPPING[Resource.SCALE_CONNECTIVITY],
